var _ = require('lodash')
var apidoc = require('apidoc-core-most')
var fs = require('fs-extra')
var path = require('path')
var winston = require('winston')
var Markdown = require('markdown-it')
var axios = require('axios')
var config = require('./config')
var empty = require('st-empty')

var PackageInfo = require('./package_info')

var defaults = {
  dest: path.join(__dirname, '../doc/'),
  template: path.join(__dirname, '../template/'),

  debug: false,
  silent: false,
  verbose: false,
  simulate: false,
  parse: false, // Only parse and return the data, no file creation.
  colorize: true,
  markdown: true,
  config: './',
  apiprivate: false,
  encoding: 'utf8',
  doc: false,
  nomost: false,
  env: 'production'
}

var app = {
  log: {},
  markdownParser: null,
  options: {}
}

// Display uncaught Exception.
process.on('uncaughtException', function (err) {
  console.error((new Date()).toUTCString() + ' uncaughtException:', err.message)
  console.error(err.stack)
  process.exit(1)
})

var configFile = {}

/**
 * Create the documentation
 *
 * @param  {Object} options See defaults and apidoc-core defaults for all options / `apidoc --help`
 * @returns {Mixed} true = ok, but nothing todo | false = error | Object with parsed data and project-informations.
 */
function createDoc(options) {
  var api
  var apidocPath = path.join(__dirname, '../')
  var markdownParser
  var packageInfo

  options = _.defaults({}, options, defaults)

  // Paths.
  options.dest = path.join(options.dest, './')
  options.template = path.join(options.template, './')

  // Line-Ending.
  if (options.lineEnding) {
    if (options.lineEnding === 'CRLF')
      options.lineEnding = '\r\n' // win32
    else if (options.lineEnding === 'CR')
      options.lineEnding = '\r' // darwin
    else
      options.lineEnding = '\n' // linux
  }

  // Options.
  app.options = options

  // Logger.
  app.log = new (winston.Logger)({
    transports: [
      new (winston.transports.Console)({
        level: app.options.debug ? 'debug' : app.options.verbose
          ? 'verbose'
          : 'info',
        silent: app.options.silent,
        prettyPrint: true,
        colorize: app.options.colorize,
        timestamp: false
      })
    ]
  })

  // Markdown Parser: enable / disable / use a custom parser.
  if (app.options.markdown === true) {
    markdownParser = new Markdown({
      breaks: false,
      html: true,
      linkify: false,
      typographer: false
    })
  } else if (app.options.markdown !== false) {
    // Include custom Parser @see MARKDOWN.md and test/fixtures/custom_markdown_parser.js
    Markdown = require(app.options.markdown) // Overwrite default Markdown.
    markdownParser = new Markdown()
  }
  app.markdownParser = markdownParser

  try {
    packageInfo = new PackageInfo(app)

    // generator information
    var json = JSON.parse(fs.readFileSync(apidocPath + 'package.json', 'utf8'))
    apidoc.setGeneratorInfos({
      name: json.name,
      time: new Date(),
      url: json.homepage,
      version: json.version
    })
    apidoc.setLogger(app.log)
    apidoc.setMarkdownParser(markdownParser)

    let configJson = packageInfo.get()
    configFile = configJson
    apidoc.setPackageInfos(configJson)

    api = apidoc.parse(app.options)

    if (api === true) {
      app.log.info('可生成文档内容为空')
      // return true
    }
    if (api === false) {
      app.log.info('匹配时发生未知错误')
      return false
    }

    if (app.options.doc) {
      app.log.info('生成Apidoc文档')
      if (app.options.parse !== true)
        createOutputFiles(api)
    }

    let _env = app.options.env || config.env || 'production'
    if (!app.options.nomost && configJson.mostId &&
      configJson.mostId[_env].length > 0) {
      app.log.info('生成Mostagile文档，环境=', _env, 'mostId=',
        configJson.mostId[_env])
      if (empty.check(api.data)) {
        injectMost(api, configJson.mostId[config.env], config.env)
      }
    }
    app.log.info('Done.')
    return api
  } catch (e) {
    app.log.error(e.message)
    if (e.stack)
      app.log.debug(e.stack)
    return false
  }
}

async function injectMost(api, mostId, env) {
  let apiJson = JSON.parse(api.data.replace(/\\r/g, ''))
  for (var api of apiJson) {
    app.log.info('here2 ', config.mostHost[env] + config.urlCreateUpdateApi)
    await axios.post(config.mostHost[env] + config.urlCreateUpdateApi,
      getMostParam(api, mostId), {
        headers: {
          Authorization: 'Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1aWQiOiI1YzQ3ZTA4YzI4YTUyYTAyNDQxMDNiMjMiLCJkYXRhIjp7InVzZXIiOiI1YmE5YTc5ZTI1YmIzMTAzYjY5NzI3MTIiLCJ0ZWFtIjoiNWFhNzY3ZDI0YWRkNjIwMDFmZmQzZjJhIiwidGVhbVVzZXIiOiI1YzQ3ZTA4YzI4YTUyYTAyNDQxMDNiMjMiLCJwcm9qZWN0IjoiNWMwNGRmOGVjMjQwYTUwMDZhNDViOTFhIn0sImlhdCI6MTU2MDQ5MzYyNiwiZXhwIjoxNTY5MTMzNjI2fQ.pljjztwW9pofegWj5bosmg_FU3PJIE9yHVEKqYC1Tqo'
        }
      })
  }
}

function getMostParam(api, mostId) {
  let { heads, requests, responses } = getMostParamField(api)
  return {
    prod: mostId,
    heads: heads,
    name: api.title,
    method: getMethodCode(api.type),
    url: api.url,
    group: `#${configFile.name}#${api.group}`,
    requests: requests,
    responses: responses,
    winExample: '',
    lossExample: '',
    remark: '',
    groupType: 'name'
  }
}

function getMostParamField(api) {
  let heads = [], requests = [], responses = []
  if (empty.check(api.header) && empty.check(api.header.fields) &&
    empty.check(api.header.fields.Header)) {
    heads = getMostParamHeader(api.header.fields.Header)
  }
  if (empty.check(api.parameter) && empty.check(api.parameter.fields) &&
    empty.check(api.parameter.fields.Parameter)) {
    requests = getMostParamRequest(api.parameter.fields.Parameter)
  }
  return { heads, requests, responses }
}

function getMostParamHeader(params) {
  let paramArray = []
  for (let param of params) {
    paramArray.push({
      name: param.field,
      content: param.description,
      describe: param.type
    })
  }
  return paramArray
}

function getMostParamRequest(params) {
  let paramArray = []
  for (let param of params) {
    paramArray.push({
      name: param.field,
      dataType: param.type,
      hasNull: param.optional,
      explain: param.description.replace(/(<p>|<\/p>)/g, ''),
      example: ''
    })
  }
  return paramArray
}

function getMethodCode(method) {
  let code = 1
  switch (method) {
    case 'post':
      code = 1
      break
    case 'put':
      code = 2
      break
    case 'get':
      code = 3
      break
    case 'delete':
      code = 4
      break
    default:
      code = 1
  }
  return code
}

/**
 * Save parsed data to files
 *
 * @param {Object[]} blocks
 * @param {Object} packageInfos
 */
function createOutputFiles(api) {
  if (app.options.simulate)
    app.log.warn('!!! Simulation !!! No file or dir will be copied or created.')

  app.log.verbose('create dir: ' + app.options.dest)
  if (!app.options.simulate)
    fs.mkdirsSync(app.options.dest)

  app.log.verbose(
    'copy template ' + app.options.template + ' to: ' + app.options.dest)
  if (!app.options.simulate)
    fs.copySync(app.options.template, app.options.dest)

  // Write api_data
  app.log.verbose('write json file: ' + app.options.dest + 'api_data.json')
  if (!app.options.simulate)
    fs.writeFileSync(app.options.dest + './api_data.json', api.data + '\n')

  app.log.verbose('write js file: ' + app.options.dest + 'api_data.js')
  if (!app.options.simulate)
    fs.writeFileSync(app.options.dest + './api_data.js',
      'define({ "api": ' + api.data + ' });' + '\n')

  // Write api_project
  app.log.verbose('write json file: ' + app.options.dest + 'api_project.json')
  if (!app.options.simulate)
    fs.writeFileSync(app.options.dest + './api_project.json',
      api.project + '\n')

  app.log.verbose('write js file: ' + app.options.dest + 'api_project.js')
  if (!app.options.simulate)
    fs.writeFileSync(app.options.dest + './api_project.js',
      'define(' + api.project + ');' + '\n')
}

module.exports = {
  createDoc: createDoc
}
